@@ -67,6 +67,8 @@
 #include "src/slurmrestd/operations.h"
 #include "src/slurmrestd/xjson.h"
 
+#include "src/slurmrestd/plugins/openapi/v0.0.36/api.h"
+
 typedef struct {
 	const char *param;
 	int optval;
@@ -216,10 +218,12 @@ const params_t job_params[] = {
 	{ "ntasks", 'n' },
 	{ "tasks_per_core", LONG_OPT_NTASKSPERCORE },
 	{ "ntasks_per_core", LONG_OPT_NTASKSPERCORE },
+	{ "ntasks_per_gpu", LONG_OPT_NTASKSPERGPU },
 	{ "tasks_per_node", LONG_OPT_NTASKSPERNODE },
 	{ "ntasks_per_node", LONG_OPT_NTASKSPERNODE },
 	{ "tasks_per_socket", LONG_OPT_NTASKSPERSOCKET },
 	{ "ntasks_per_socket", LONG_OPT_NTASKSPERSOCKET },
+	{ "ntasks_per_tres", LONG_OPT_NTASKSPERTRES },
 	{ "temporary_disk_per_node", LONG_OPT_TMP },
 	{ "test_only", LONG_OPT_TEST_ONLY },
 	{ "thread_specification", LONG_OPT_THREAD_SPEC },
@@ -375,6 +379,11 @@ static int _fill_job_desc_from_sbatch_opts(slurm_opt_t *opt,
 		desc->min_nodes = 0;
 	if (opt->ntasks_per_node)
 		desc->ntasks_per_node = opt->ntasks_per_node;
+	if (opt->ntasks_per_tres != NO_VAL)
+		desc->ntasks_per_tres = opt->ntasks_per_tres;
+	else if (opt->ntasks_per_gpu != NO_VAL)
+		desc->ntasks_per_tres = opt->ntasks_per_gpu;
+
 	/* Disable sending uid/gid as it is handled by auth layer */
 	/* desc->user_id = opt->uid; */
 	/* desc->group_id = opt->gid; */
@@ -591,7 +600,7 @@ static job_desc_msg_t *_parse_job_desc(const data_t *job, data_t *errors,
 		goto cleanup;
 	}
 
-	if (!req->environment || !req->env_size) {
+	if (!update_only && (!req->environment || !req->env_size)) {
 		/*
 		 * Jobs provided via data must have their environment
 		 * setup or they will simply be rejected. Error now instead of
@@ -752,20 +761,9 @@ static job_parse_list_t _parse_job_list(const data_t *jobs, char *script,
 	return rc;
 }
 
-/* dump job integer list as data list
- * IN ptr to integer list (-1 is terminator)
- * IN d ptr to data to populate
- */
-static void _dump_job_int32_list(const int32_t *ptr, data_t *d)
-{
-	// based on _print_job_exc_node_inx
-	data_set_list(d);
-	while (*ptr != -1)
-		data_set_int(data_list_append(d), *(ptr++));
-}
-
 static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 {
+	xassert(data_get_type(jd) == DATA_TYPE_NULL);
 	data_set_dict(jd);
 	data_set_string(data_key_set(jd, "account"), job->account);
 	data_set_int(data_key_set(jd, "accrue_time"), job->accrue_time);
@@ -1015,11 +1013,6 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	else
 		data_set_int(data_key_set(jd, "nice"), job->nice - NICE_OFFSET);
 	/* node_index intentionally omitted */
-	if (job->node_inx)
-		_dump_job_int32_list(job->node_inx,
-				     data_key_set(jd, "node_index"));
-	else
-		data_set_null(data_key_set(jd, "node_index"));
 	if (job->ntasks_per_core == NO_VAL16 ||
 	    job->ntasks_per_core == INFINITE16)
 		data_set_null(data_key_set(jd, "tasks_per_core"));
@@ -1039,7 +1032,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 		data_set_int(data_key_set(jd, "cpus"), job->num_cpus);
 	else
 		data_set_null(data_key_set(jd, "cpus"));
-	data_set_int(data_key_set(jd, "nodes"), job->num_nodes);
+	data_set_int(data_key_set(jd, "node_count"), job->num_nodes);
 	if (job->num_tasks != NO_VAL && job->num_tasks != INFINITE)
 		data_set_int(data_key_set(jd, "tasks"), job->num_tasks);
 	else
@@ -1062,7 +1055,8 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 		data_set_null(data_key_set(jd, "memory_per_cpu"));
 	}
 	data_set_int(data_key_set(jd, "minimum_cpus_per_node"), job->pn_min_cpus);
-	data_set_int(data_key_set(jd, "pn_min_tmp_disk"), job->pn_min_tmp_disk);
+	data_set_int(data_key_set(jd, "minimum_tmp_disk_per_node"),
+		     job->pn_min_tmp_disk);
 	/* power_flags intentionally omitted */
 	data_set_int(data_key_set(jd, "preempt_time"), job->preempt_time);
 	data_set_int(data_key_set(jd, "pre_sus_time"), job->pre_sus_time);
@@ -1091,12 +1085,7 @@ static data_t *dump_job_info(slurm_job_info_t *job, data_t *jd)
 	data_set_string(data_key_set(jd, "qos"), job->qos);
 	data_set_bool(data_key_set(jd, "reboot"), job->reboot);
 	data_set_string(data_key_set(jd, "required_nodes"), job->req_nodes);
-	if (job->req_node_inx)
-		_dump_job_int32_list(job->req_node_inx,
-				     data_key_set(jd,
-						  "requested_node_by_index"));
-	else
-		data_set_null(data_key_set(jd, "requested_node_by_index"));
+	/* skipping req_node_inx */
 	data_set_bool(data_key_set(jd, "requeue"), job->requeue);
 	data_set_int(data_key_set(jd, "resize_time"), job->resize_time);
 	data_set_int(data_key_set(jd, "restart_cnt"), job->restart_cnt);
@@ -1199,18 +1188,21 @@ static int _op_handler_jobs(const char *context_id,
 {
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
+	(void) populate_response_format(resp);
+	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
 
-	data_set_list(resp);
 	debug4("%s: jobs handler called by %s", __func__, context_id);
 
 	rc = slurm_load_jobs((time_t)NULL, &job_info_ptr,
 			     SHOW_ALL|SHOW_DETAIL);
 
 	if (rc == SLURM_SUCCESS && job_info_ptr &&
-	    job_info_ptr->record_count)
-		for (size_t i = 0; i < job_info_ptr->record_count; ++i)
+	    job_info_ptr->record_count) {
+		for (size_t i = 0; i < job_info_ptr->record_count; ++i) {
 			dump_job_info(job_info_ptr->job_array + i,
-				      data_list_append(resp));
+				      data_list_append(jobs));
+		}
+	}
 
 	slurm_free_job_info_msg(job_info_ptr);
 
@@ -1241,13 +1233,16 @@ static int _handle_job_get(const char *context_id, http_request_method_t method,
 	int rc = SLURM_SUCCESS;
 	job_info_msg_t *job_info_ptr = NULL;
 	rc = slurm_load_job(&job_info_ptr, job_id, SHOW_ALL|SHOW_DETAIL);
+	data_t *jobs = data_set_list(data_key_set(resp, "jobs"));
 
-	if (rc == SLURM_SUCCESS && job_info_ptr &&
-	    job_info_ptr->record_count)
-		for (size_t i = 0; i < job_info_ptr->record_count; ++i)
-			dump_job_info(job_info_ptr->job_array + i, resp);
-	else
+	if (!rc && job_info_ptr && job_info_ptr->record_count) {
+		for (size_t i = 0; i < job_info_ptr->record_count; ++i) {
+			dump_job_info(job_info_ptr->job_array + i,
+				      data_list_append(jobs));
+		}
+	} else {
 		_job_error("%s: unknown job %d", __func__, job_id);
+	}
 
 	slurm_free_job_info_msg(job_info_ptr);
 
@@ -1260,13 +1255,19 @@ static int _handle_job_delete(const char *context_id,
 			      data_t *resp, const uint32_t job_id,
 			      data_t *const errors, const int signal)
 {
-	int rc = slurm_kill_job(job_id, signal, KILL_FULL_JOB);
+	if (slurm_kill_job(job_id, signal, KILL_FULL_JOB)) {
+		int rc;
 
-	if (rc)
-		_job_error("%s: unable to kill job %d with signal %d: %m",
-			   __func__, job_id, signal);
+		/* Already signaled jobs are considered a success here */
+		if (errno == ESLURM_ALREADY_DONE)
+			return SLURM_SUCCESS;
 
-	return rc;
+		_job_error("%s: unable to kill job %d with signal %d: %s",
+			   __func__, job_id, signal, slurm_strerror(errno));
+		return rc;
+	}
+
+	return SLURM_SUCCESS;
 }
 
 static int _handle_job_post(const char *context_id,
@@ -1326,8 +1327,7 @@ static int _op_handler_job(const char *context_id, http_request_method_t method,
 	int rc = SLURM_SUCCESS;
 	data_t *data_jobid;
 	uint32_t job_id = 0;
-	data_t *errors = data_set_list(
-		data_key_set(data_set_dict(resp), "errors"));
+	data_t *errors = populate_response_format(resp);
 	debug4("%s: job handler %s called by %s with tag %d",
 	       __func__, get_http_method_string(method), context_id, tag);
 
@@ -1427,7 +1427,7 @@ static int _op_handler_submit_job_post(const char *context_id,
 		if (!jobs) {
 			error("%s: [%s] missing job specification field",
 			      __func__, context_id);
-			rc = ESLURM_INVALID_JOB_ID;
+			rc = ESLURM_REST_INVALID_JOBS_DESC;
 		} else if ((jobs_rc = _parse_job_list(jobs, script, errors,
 						      false))
 				   .rc) {
@@ -1471,6 +1471,10 @@ static int _op_handler_submit_job_post(const char *context_id,
 			data_set_string(data_key_set(d, "step_id"),
 					"EXTERN");
 			break;
+		case SLURM_INTERACTIVE_STEP:
+			data_set_string(data_key_set(d, "step_id"),
+					"INTERACTIVE");
+			break;
 		default:
 			data_set_int(data_key_set(d, "step_id"),
 				     resp->step_id);
@@ -1495,8 +1499,8 @@ finish:
 		data_set_string(data_key_set(error, "error"),
 				slurm_strerror(rc));
 
-		debug5("%s: [%s] job submission failed with %d: %m",
-		       __func__, context_id, errno);
+		debug5("%s: [%s] job submission failed with %d: %s",
+		       __func__, context_id, rc, slurm_strerror(rc));
 	}
 
 	slurm_free_submit_response_response_msg(resp);
@@ -1512,8 +1516,7 @@ static int _op_handler_submit_job(const char *context_id,
 				  rest_auth_context_t *auth)
 {
 	int rc = SLURM_SUCCESS;
-	data_t *errors = data_set_list(
-		data_key_set(data_set_dict(resp), "errors"));
+	data_t *errors = populate_response_format(resp);
 
 	debug4("%s: job submit handler %s called by %s with tag %d",
 	       __func__, get_http_method_string(method), context_id, tag);
@@ -1566,11 +1569,11 @@ extern void init_op_jobs(void)
 			      __func__);
 	}
 
-	bind_operation_handler("/slurm/v0.0.35/jobs/", _op_handler_jobs,
+	bind_operation_handler("/slurm/v0.0.36/jobs/", _op_handler_jobs,
 			       URL_TAG_JOBS);
-	bind_operation_handler("/slurm/v0.0.35/job/{job_id}", _op_handler_job,
+	bind_operation_handler("/slurm/v0.0.36/job/{job_id}", _op_handler_job,
 			       URL_TAG_JOB);
-	bind_operation_handler("/slurm/v0.0.35/job/submit",
+	bind_operation_handler("/slurm/v0.0.36/job/submit",
 			       _op_handler_submit_job, URL_TAG_JOB_SUBMIT);
 }
 
